import requests
from cad_tickers.exchanges.classes import CSESedarFilings, CSETicker
from typing import Union
from cad_tickers.util import make_cse_path


def get_cse_ticker_data(ticker, get_dict: bool = True) -> Union[CSETicker, dict]:
    """
    Parameters:
        ticker - stock ticker for the cse without exchange (ex, CMC)
        get_dict - flag to get dict
    Returns:
        cse_ticker - dict or python class containing ticker data
    """
    ticker_url = f"https://webapi.thecse.com/trading/listed/securities/{ticker}.json"
    r = requests.get(ticker_url)
    data = r.json()

    if get_dict == True:
        return data
    return CSETicker(**data)


def get_cse_sedar_docs(
    cse_data: Union[dict, CSETicker]
) -> Union[dict, CSESedarFilings]:
    """
    Parameters:
      cse_data - information for a ticker when it is loaded on the thecse website
    Returns:
      filings - class or dict with properties :class:`cad_tickers.exchanges.classes.CSESedarFilings`
    """
    if isinstance(cse_data, CSETicker) or isinstance(cse_data, dict):
        if isinstance(cse_data, CSETicker):
            metatdata = cse_data.metatdata
        else:
            if "metatdata" in cse_data.keys():
                metatdata = cse_data.get("metatdata")
        sedar_filings_url = metatdata.get("sedar_filings")
        r = requests.get(sedar_filings_url)
        sedar_filings = r.json()
        return sedar_filings
    else:
        raise Exception("data misformatted must be dict or cse ticker")


if __name__ == "__main__":
    import json
    import os

    cmc_path = os.path.join("tests", "sample_data", "CMC.json")
    cmc_json = {}
    try:
        with open(cmc_path) as f:
            cmc_json = json.load(f)
    except Exception as e:
        print(e)
    fillings = get_cse_sedar_docs(cmc_json)
    cmc = get_cse_ticker_data("CMC")



from dataclasses import dataclass
from time import time

@dataclass
class SearchParams:
    '''Search Params for Ceo.ca intended to get spiels.'''
    channel: str = '@newswire'
    filter_terms: str = 'APHA'
    filter_top: int = 100
    load_more: str = 'top'
    original_scroll_height: str = 0
    # unix timestamp in millseconds
    until: int = int(time() * 1000) 




"""
Exchanges
----------

Downloading and cleaning data from the cse and tsx exchanges
"""

from cad_tickers.exchanges.cse import (
    get_cse_files,
    get_cse_tickers_df,
    get_description_for_url,
    get_all_cse_tickers,
)
from cad_tickers.exchanges.tsx import (
    get_all_tickers_data,
    get_all_tsx_tickers,
    get_ticker_data,
    get_tickers,
    get_tsx_tickers,
    gql_data,
)
from cad_tickers.exchanges.classes import CSETicker, CSESedarFilings

from cad_tickers.exchanges.all_tickers import mk_full_tickers



import requests
import time
from cad_tickers.news.ceo import SearchParams, ceo_url, \
    news_link_from_spiel, \
    params_to_dict

from cad_tickers.news.ceo.utils import art_pixel_height, earlier_timestamp, module_logger
from typing import Tuple, List
from dataclasses import replace
# get spiels from 
def get_spiels(params: dict)-> dict:
    """Simple function to get ceo.ca spiels"""
    fetch_url = f"{ceo_url}/api/get_spiels"
    r = requests.get(fetch_url, params=params)
    # print(r.url)
    data = r.json() 
    return data

def extract_urls(spiels: List[dict])-> Tuple[list, str]:
    """Valid List Items from ceo"""
    spiels_list = []
    if len(spiels) == 0:
        return [], ''
    for spiel in spiels:
        url = news_link_from_spiel(spiel)
        if url != None:
            spiels_list.append(url)
        else:
            raise Exception('URL SHOULD NOT BE NULL')
    # using list comprehension 
    # to remove None values in list 
    # spiels_list = [i for i in spiels_list if i] 
    return spiels_list, spiels[0].get('timestamp')

def get_new_items(ticker: str, channel='@newswire', max_iterations = 60, until = 0):
    """Gets news items from ceo using ticker

        Parameters:
            ticker - stock ticker, for example APHA
            channel - can be @newswire, @thenewswire 
            max_iterations - max number of requests to ceo.ca
    """
    sp = SearchParams(channel=channel, filter_terms=ticker)
    scroll_height = 0
    data_urls = []
    blank_fetchs = 0
    for i in range(max_iterations):
        if until != 0:
            # update object with new timestamp
            sp = replace(sp, **{
                'until': until, 
                'original_scroll_height': scroll_height
            })
            module_logger.warning(f'Update timestamp: {until}')
        params = params_to_dict(sp)
        data = get_spiels(params)
        # standard delay to not get blocked
        time.sleep(0.5)
        spiels: list = data.get('spiels', [])
        urls, new_timestamp = extract_urls(spiels)
        if len(urls) == 0:
            # push back timestamp by 2 months
            until = earlier_timestamp(until, 60)
            blank_fetchs = blank_fetchs + 1
            # nothing new for about a year
            # stop fetching data
            if blank_fetchs >= 7:
                break
            continue
        else:
            blank_fetchs = 0
            until = new_timestamp
        scroll_height += art_pixel_height * len(urls)
        data_urls = [*data_urls, *urls]
    return data_urls


# Function that selectors any article class and then returns all the text, alternatively can use sumpy for a well
# formed pages like ceo.ca
if __name__ == "__main__":
    data_urls = get_new_items('APHA', until=1609217267394)


import re
import datetime
import logging
from dataclasses import asdict
from typing import Type, Union
from cad_tickers.news.ceo import SearchParams

# create logger
module_logger = logging.getLogger(__name__) 

ceo_url = "https://ceo.ca"
art_pixel_height = 96
def params_to_dict(sp: Type[SearchParams]) -> dict:
    """utility function to get query parameters"""
    data = asdict(sp)
    data["filters[terms]"] = data.pop("filter_terms")
    data["filters[top]"] = data.pop("filter_top")
    return data

def news_link_from_spiel(spiel: dict)-> Union[str, None]:
    spiel = spiel.get('spiel')
    try:
        found = re.search('@[a-zA-Z0-9/-]+', spiel)
        return f"{ceo_url}/{found.group(0)}"
    except AttributeError as e:
        logging.warning(e)
        return None

def earlier_timestamp(timestamp: int, days: int=90)-> int:
    """Update timestamp and return timestamp in millseconds"""
    timestamp = float(timestamp * 0.001)
    orig = datetime.datetime.fromtimestamp(timestamp)
    new = orig - datetime.timedelta(days=days)
    return int(new.timestamp() * 1000)




class GQL:
    """
    quote_by_symbol_query:

    .. _quote_by_symbol_query:

    graphql properties for `getQuoteBySymbol` are
      * symbol
      *  name
      *  price
      *  priceChange
      *  percentChange
      *  exchangeName
      *  exShortName
      *  exchangeCode
      *  marketPlace
      *  sector
      *  industry
      *  volume
      *  openPrice
      *  dayHigh
      *  dayLow
      *  MarketCap
      *  MarketCapAllClasses
      *  peRatio
      *  prevClose
      *  dividendFrequency
      *  dividendYield
      *  dividendAmount
      *  dividendCurrency
      *  beta
      *  eps
      *  exDividendDate
      *  shortDescription
      *  longDescription
      *  website
      *  email
      *  phoneNumber
      *  fullAddress
      *  employees
      *  shareOutStanding
      *  totalDebtToEquity
      *  totalSharesOutStanding
      *  sharesESCROW
      *  vwap
      *  dividendPayDate
      *  weeks52high
      *  weeks52low
      *  alpha
      *  averageVolume10D
      *  averageVolume30D
      *  averageVolume50D
      *  priceToBook
      *  priceToCashFlow
      *  returnOnEquity
      *  returnOnAssets
      *  day21MovingAvg
      *  day50MovingAvg
      *  day200MovingAvg
      *  dividend3Years
      *  dividend5Years
      *  datatype
      *  __typename
    """

    quote_by_symbol_query = """query getQuoteBySymbol($symbol: String, $locale: String) {
      getQuoteBySymbol(symbol: $symbol, locale: $locale) {
        symbol
        name
        price
        priceChange
        percentChange
        exchangeName
        exShortName
        exchangeCode
        marketPlace
        sector
        industry
        volume
        openPrice
        dayHigh
        dayLow
        MarketCap
        MarketCapAllClasses
        peRatio
        prevClose
        dividendFrequency
        dividendYield
        dividendAmount
        dividendCurrency
        beta
        eps
        exDividendDate
        shortDescription
        longDescription
        website
        email
        phoneNumber
        fullAddress
        employees
        shareOutStanding
        totalDebtToEquity
        totalSharesOutStanding
        sharesESCROW
        vwap
        dividendPayDate
        weeks52high
        weeks52low
        alpha
        averageVolume10D
        averageVolume30D
        averageVolume50D
        priceToBook
        priceToCashFlow
        returnOnEquity
        returnOnAssets
        day21MovingAvg
        day50MovingAvg
        day200MovingAvg
        dividend3Years
        dividend5Years
        datatype
        __typename
      }
    }
  """

    quote_by_symbol_payload = {
        "operationName": "getQuoteBySymbol",
        "variables": {"locale": "en"},
        "query": quote_by_symbol_query,
    }

    get_company_news_events_query = """query getNewsAndEvents(
      $symbol: String!,
      $page: Int!,
      $limit: Int!,
      $locale: String!
    ) { 
      news: getNewsForSymbol(
        symbol: $symbol,
        page: $page,
        limit: $limit,
        locale: $locale
      ) {
        headline
        datetime
        source
        newsid
        summary
        __typename
      }
      events: getUpComingEventsForSymbol(symbol: $symbol, locale: $locale) {
        title
        date
        status
        type
        __typename
        }
      }
    """

    get_company_news_events_payload = {
        "operationName": "getNewsAndEvents",
        "variables": {
            "symbol": "ART",
            "page": 1,
            "limit": 100,
            "locale": "en"
        },
        "query": get_company_news_events_query,
    }

    get_company_filings_query = """query getCompanyFilings(
      $symbol: String!
      $fromDate: String
      $toDate: String
      $limit: Int
    ) {
      filings: getCompanyFilings(
        symbol: $symbol
        fromDate: $fromDate
        toDate: $toDate
        limit: $limit
      ) {
        size
        filingDate
        description
        name
        urlToPdf
        __typename
      }
    }"""

    # Replace the fromDate and toDate variables, or default them to
    # the current month
    get_company_filings_payload = {
        "operationName": "getCompanyFilings",
        "variables": {
            "symbol": "ART",
            "fromDate": "2020-09-01",
            "toDate": "2020-09-30",
            "limit": 100,
        },
        "query": get_company_filings_query,
    }
